#!/bin/sh
###################################################################
#
# About: This script is a wrapper to pwsdbsetup.jar to set up the
#        Platform Web Services PostgreSQL database.
#
#        Use -h to list the required environment variables that
#        you need to set.
#
#        If you do not specify any options, the script will create
#        a PostgreSQL database 'DB_NAME' and execute 3 SQL scripts
#        to create the database tables.
#        
#        Use -upgrade to upgrade a PostgreSQL database 'DB_NAME' to a newer version.
#
#        Use -U to uninstall the database.
#
# Error Codes: 0 = Success
#              1 = Fail with error message output
#
# Error Messages:
#
# Error (pwsdbsetup.sh): JAVA_HOME environment variable not specified. Make sure the JRE is installed and JAVA_HOME is set
#
# Error (pwsdbsetup.sh): DB_ADMIN_USER environment variable not specified. Set DB_ADMIN_USER to the user name of the database administrator.
#
# Error (pwsdbsetup.sh): DB_CONNECTION_STRING environment variable not specified. Set DB_CONNECTION_STRING to the jdbc database connection string. For example: jdbc:postgresql://hostname:5432
#
# Error (pwsdbsetup.sh): JRE 1.6 or higher is required. The current JRE version is <current version> in <JAVA_HOME>.
#
# Error (pwsdbsetup.sh): JDBC_DRIVER_JAR environment variable not specified. Set JDBC_DRIVER_JAR to the path of the JDBC driver jar. For example: /usr/lib/postgresql-9.2-1002.jdbc4.jar 
#
# Error (pwsdbsetup.sh): pwsdbsetup.jar is missing from the current directory, <pwd>. Check your installation package.
#
# Error (pwsdbsetup.sh): DBschema/PostgreSQL/create_schema.sql is missing from the current directory, <pwd>. Check your installation package. 
#
# Error (pwsdbsetup.sh): DBschema/PostgreSQL/create_pac_schema.sql is missing from the current directory, <pwd>. Check your installation package.
#
# Error (pwsdbsetup.sh): DBschema/PostgreSQL/init.sql is missing from the current directory, <pwd>. Check your installation package.
#
# Error (pwsdbsetup.sh): The upgrade script in DBschema/PostgreSQL/Patch/ is missing from the current directory, <pwd>. Check your installation package. 
#
# Error (pwsdbsetup.sh): Unable to connect to the database with the set environment variables. Run pwsdbsetup.sh -h for required environment variables. 
#
###################################################################

print_usage(){
    echo "----------------------------------------"
    echo "Usage: pwsdbsetup.sh"
    echo "       pwsdbsetup.sh -h"
    echo "       pwsdbsetup.sh -upgrade [<PWS_NEW_VERSION>]"
    echo "       pwsdbsetup.sh -U"
    echo "Without arguments the database will be installed"
    echo "-h         Display the required environment variables"
    echo "-upgrade   Update current PWS version's database to the specified newer PWS version's database"
    echo "-U         Uninstall the database specified by environment variable DB_NAME"
    echo "-----------------------------------------"
    echo "[REQUIRED] Environment variables to  be set:";
    echo "JDBC_DRIVER_JAR      - Full path to the JDBC driver jar.";
    echo "                       Example: /usr/lib/postgresql-9.2-1002.jdbc4.jar";
    echo "DB_CONNECTION_STRING - JDBC connection string to connect to database. Do not include the database name.";
    echo "                       Example: jdbc:postgresql://hostname:5432";
    echo "DB_ADMIN_USER        - User name of the database administrator";
    echo "                       Example: postgres";
    echo "DB_ADMIN_PASSWORD    - Password of the data base administrator";
    echo "DB_PWS_USER          - Database user whom PWS will use to connect and access the database";
    echo "                       Example: pacuser";
    echo "DB_NAME              - Database name to be installed for PWS to use. Default value 'pac'";
    echo "PWS_NEW_VERSION      - PWS version must be newer than the current version. The PWS database will be updated to the version specified by PWS_NEW_VERSION.";
    echo "                       Example: pwsdbsetup.sh -upgrade 9.1.3 will upgrade the PWS database to version 9.1.3";
    exit 0;
}

# Check if -h or invalid argument was used
if [ "$1" = "-h" -o "$1" != "-U" -a "$1" != "-upgrade" -a "$1" != "" ]; then
    print_usage
fi;

# Check if -upgrade or invalid argument was used
if [ "$1" != "" -a "$1" = "-upgrade" ]; then
    if [ "$2" = "" ]; then
        print_usage
    fi;
fi;

# Check if none of the variables are set
if [ "$JDBC_DRIVER_JAR" = "" -a "$DB_CONNECTION_STRING" = "" -a "$DB_ADMIN_USER" = "" -a "$DB_ADMIN_PASSWORD" = "" -a "$DB_PWS_USER" = "" ]; then
    print_usage
fi;

if [ "$1" = "-upgrade" ]; then
    UPGRADE=1;
else
    UPGRADE=0;
fi

if [ "$1" = "-U" ]; then
    UNINSTALL=1;
else
    UNINSTALL=0;
fi

# Check JAVA_HOME
if [ "$JAVA_HOME" = "" ]; then
    echo "Error (pwsdbsetup.sh): JAVA_HOME environment variable not specified. Make sure the JRE is installed and JAVA_HOME is set.";
    exit 1;
fi;

#Check java version > 1.6
VERSION=`$JAVA_HOME/bin/java -version 2>&1 | head -n 1 | cut -d\" -f 2`
MAJOR=`echo $VERSION |cut -d'.' -f1`
MINOR=`echo $VERSION |cut -d'.' -f2`
if [ $MAJOR -lt 1 -o $MINOR -lt 6 ]; then
    echo "Error (pwsdbsetup.sh): JRE 1.6 or higher is required. The current JRE version is $MAJOR.$MINOR in $JAVA_HOME.";
    exit 1;
fi;

# Check for files
if [ ! -f "pwsdbsetup.jar" ]; then
    echo "Error (pwsdbsetup.sh): pwsdbsetup.jar is missing from the current directory, `pwd`. Check your installation package.";
    exit 1;
fi;
if [ ! -f "DBschema/PostgreSQL/create_schema.sql" ]; then
    echo "Error (pwsdbsetup.sh): DBschema/PostgreSQL/create_schema.sql is missing from the current directory, `pwd`. Check your installation package.";
    exit 1;
fi;
if [ ! -f "DBschema/PostgreSQL/create_pac_schema.sql" ]; then
    echo "Error (pwsdbsetup.sh): DBschema/PostgreSQL/create_pac_schema.sql is missing from the current directory, `pwd`. Check your installation package.";
    exit 1;
fi;
if [ ! -f "DBschema/PostgreSQL/init.sql" ]; then
    echo "Error (pwsdbsetup.sh): DBschema/PostgreSQL/init.sql is missing from the current directory, `pwd`. Check your installation package.";
    exit 1;
fi;

# Adding defaulting of DB_ADMIN_USER to postgres if blank
if [ "$DB_ADMIN_USER" = "" ]; then
    DB_ADMIN_USER=postgres;
fi;

# Check for env vars
if [ "$JDBC_DRIVER_JAR" = "" ]; then
    echo "Error (pwsdbsetup.sh): JDBC_DRIVER_JAR environment variable not specified. Set JDBC_DRIVER_JAR to the absolute path to the JDBC driver jar file.";
    exit 1;
fi;
if [ "$DB_CONNECTION_STRING" = "" ]; then
    echo "Error (pwsdbsetup.sh): DB_CONNECTION_STRING environment variable not specified. Set DB_CONNECTION_STRING to the jdbc database connection string. For example: jdbc:postgresql://hostname:5432";
    exit 1;
fi;
if [ "$DB_ADMIN_USER" = "" ]; then
    echo "Error (pwsdbsetup.sh): DB_ADMIN_USER environment variable not specified. Set DB_ADMIN_USER to the user name of the database administrator.";
    exit 1;
fi;
if [ "$DB_ADMIN_PASSWORD" = "" ]; then
    echo "Error (pwsdbsetup.sh): DB_ADMIN_PASSWORD environment variable not specified. Set DB_ADMIN_PASSWORD to the password for the database administrator.";
    exit 1;
fi;
if [ "$DB_PWS_USER" = "" ]; then
    echo "Error (pwsdbsetup.sh): DB_PWS_USER environment variable not specified. Set DB_PWS_USER to database user whom PWS will use to connect and access the database.";
    exit 1;
fi;
if [ "$DB_NAME" = "" ]; then
    DB_NAME=pac 
fi;

# Test if the database connection string can be used to connect to db
check_connection() {
    $JAVA_HOME/bin/java -classpath "$JDBC_DRIVER_JAR:./pwsdbsetup.jar" pwsdbsetup "$DB_CONNECTION_STRING/postgres" "test" $DB_ADMIN_USER
}
# Test if the database exists
check_db_exists() {
    $JAVA_HOME/bin/java -classpath "$JDBC_DRIVER_JAR:./pwsdbsetup.jar" pwsdbsetup "$DB_CONNECTION_STRING/$DB_NAME" "test" $DB_ADMIN_USER > /dev/null 2>&1
}

typeset currentPWSVersion=0;
get_current_pws_version(){
    retset=`$JAVA_HOME/bin/java -classpath "$JDBC_DRIVER_JAR:./pwsdbsetup.jar" pwsdbsetup "$DB_CONNECTION_STRING/$DB_NAME" "select * from PWS_LSF_CFG_ISAPPLYING;" "$DB_ADMIN_USER"; 2>&1`
    not_found_str="pws_lsf_cfg_isapplying"
    if [ "$retset" != "$not_found_str" ];then
        currentPWSVersion=91
    else
        currentPWSVersion=913
    fi
}

check_connection
if [ $? -eq 1 ]; then
    echo "Error (pwsdbsetup.sh): Unable to connect to the database with the set environment variables. Run pwsdbsetup.sh -h for required environment variables.";
    exit 1;
fi;

check_db_exists
if [ $? -eq 0 -a $UNINSTALL -eq 0 -a $UPGRADE -eq 0 ]; then
    echo "Error (pwsdbsetup.sh): Database with name '$DB_NAME' already exists. Change the DB_NAME field in the pwsdbsetup.sh script to another database name or uninstall/upgrade existing PWS database.";
    exit 1;
fi;

check_db_exists
if [ $? -eq 1 -a $UNINSTALL -eq 1 ]; then
    echo "Error (pwsdbsetup.sh): Database with name '$DB_NAME' does not exist. Unable to uninstall database.";
    exit 1;
fi;
check_db_exists
if [ $? -eq 1 -a $UPGRADE -eq 1 ]; then
    echo "Error (pwsdbsetup.sh): Database with name '$DB_NAME' does not exist. Unable to upgrade the database.";
    exit 1;
fi;

# Check if Uninstall
if [ $UNINSTALL -eq 1 ]; then
    $JAVA_HOME/bin/java -classpath "$JDBC_DRIVER_JAR:./pwsdbsetup.jar" pwsdbsetup "$DB_CONNECTION_STRING/postgres" "drop database \"$DB_NAME\";" "$DB_ADMIN_USER";
    if [ $? -ne 0 ]; then
        echo "Error (pwsdbsetup.sh): Drop database with name '$DB_NAME' failed. Manually drop database $DB_NAME from database."
        exit $?
    fi;
    exit 0;
fi;

# Check if Upgrade
if [ $UPGRADE -eq 1 ]; then
    get_current_pws_version
    echo "INFO (pwsdbsetup.sh): PWS current version is $currentPWSVersion"
    tempVersion=$2
    newPWSVersion=`echo ${tempVersion//./""}`
    echo "INFO (pwsdbsetup.sh): PWS new version is $newPWSVersion"
    if [ $currentPWSVersion -ne 0 -a $currentPWSVersion -ge $newPWSVersion ]; then
        echo "Error (pwsdbsetup.sh): To be upgraded PWS version must be newer than current PWS version.";
        print_usage
    fi;

    # Check files and upgrade DB if NOT checking out the currrent PWS version
    if [ $currentPWSVersion -eq 0 ]; then
        echo "INFO (pwsdbsetup.sh): All upgrade scripts will be applied to upgrade database."
        dir_name="DBschema/PostgreSQL/Patch"
        all=`ls ${dir_name}`
        pws_sql_str="pws"
        if [ "$all" = "" ]; then
            echo "Error (pwsdbsetup.sh): No SQL scripts found in DBschema/PostgreSQL/Patch. Check your installation package."
            exit 1;
        fi
        for i in $all
        do
            # For later rollbacking check
            if [ "$pws_sql_str" = "${i/$pws_sql_str}" ]; then 
                if [ ! -f "DBschema/PostgreSQL/Patch/$i" ]; then
                    echo "Error (pwsdbsetup.sh): DBschema/PostgreSQL/Patch/$i is missing from the current directory, `pwd`. Check your installation package.";
                    exit 1;
                fi;
                echo "INFO (pwsdbsetup.sh): Execute SQL file [$i] to upgrade database."
                $JAVA_HOME/bin/java -classpath "$JDBC_DRIVER_JAR:./pwsdbsetup.jar" pwsdbsetup "$DB_CONNECTION_STRING/$DB_NAME" "DBschema/PostgreSQL/Patch/$i" "$DB_ADMIN_USER";
                if [ $? -ne 0 ]; then
                    echo "Error (pwsdbsetup.sh): Upgrade database with name '$DB_NAME' failed. Manually upgrade database $DB_NAME from database."
                    exit $?
                fi;
                exit 0;
            fi
        done
        exit 0;
    fi;

    # Check files and upgrade DB if checking out the currrent PWS version
    if [ ! -f "DBschema/PostgreSQL/Patch/pws${currentPWSVersion}_to_${newPWSVersion}.sql" ]; then
        echo "Error (pwsdbsetup.sh): DBschema/PostgreSQL/Patch/pws${currentPWSVersion}_to_${newPWSVersion}.sql is missing from the current directory, `pwd`. Check your installation package.";
        exit 1;
    fi;
    $JAVA_HOME/bin/java -classpath "$JDBC_DRIVER_JAR:./pwsdbsetup.jar" pwsdbsetup "$DB_CONNECTION_STRING/$DB_NAME" "DBschema/PostgreSQL/Patch/pws${currentPWSVersion}_to_${newPWSVersion}.sql" "$DB_ADMIN_USER";
    if [ $? -ne 0 ]; then
        echo "Error (pwsdbsetup.sh): Upgrade database with name '$DB_NAME' failed. Manually upgrade database $DB_NAME using the upgrade script, DBschema/PostgreSQL/Patch/pws${currentPWSVersion}_to_${newPWSVersion}.sql."
        exit $?
    fi;
    exit 0;
fi;

# Setup the pac database for PWS postgresql
$JAVA_HOME/bin/java -classpath "$JDBC_DRIVER_JAR:./pwsdbsetup.jar" pwsdbsetup "$DB_CONNECTION_STRING/postgres" "create database \"$DB_NAME\"  with encoding 'UTF8' owner \"$DB_PWS_USER\" ;" "$DB_ADMIN_USER";
if [ $? -ne 0 ]; then
    echo "Error (pwsdbsetup.sh): Creation of the database $DB_NAME failed."
    exit $?
fi;

$JAVA_HOME/bin/java -classpath "$JDBC_DRIVER_JAR:./pwsdbsetup.jar" pwsdbsetup "$DB_CONNECTION_STRING/$DB_NAME" "DBschema/PostgreSQL/create_schema.sql" "$DB_ADMIN_USER";
if [ $? -ne 0 ]; then
    echo "Error (pwsdbsetup.sh): Execution of the create_schema sql script failed."
    exit $?
fi;

$JAVA_HOME/bin/java -classpath "$JDBC_DRIVER_JAR:./pwsdbsetup.jar" pwsdbsetup "$DB_CONNECTION_STRING/$DB_NAME" "DBschema/PostgreSQL/create_pac_schema.sql" "$DB_ADMIN_USER";
if [ $? -ne 0 ]; then
    echo "Error (pwsdbsetup.sh): Execution of the create_pac_schema sql script failed."
    exit $?
fi;

$JAVA_HOME/bin/java -classpath "$JDBC_DRIVER_JAR:./pwsdbsetup.jar" pwsdbsetup "$DB_CONNECTION_STRING/$DB_NAME" "DBschema/PostgreSQL/init.sql" "$DB_ADMIN_USER";
if [ $? -ne 0 ]; then
    echo "Error (pwsdbsetup.sh): Execution of the init sql script failed."
    exit $?
fi;
exit 0;
